"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// Masks
var OP_MASK = 0xfc000000;
var RS_MASK = 0x03e00000;
var RT_MASK = 0x001f0000;
var RD_MASK = 0x0000f800;
var SH_MASK = 0x000007c0;
var FUNC_MASK = 0x0000003f;
var IMM_MASK = 0x0000ffff;
var ADDR_MASK = 0x03ffffff;
// Shifts
var OP_SHIFT = 26;
var RS_SHIFT = OP_SHIFT - 5;
var RT_SHIFT = RS_SHIFT - 5;
var RD_SHIFT = RT_SHIFT - 5;
var SH_SHIFT = RD_SHIFT - 5;
var FUNC_SHIFT = SH_SHIFT - 6;
var IMM_SHIFT = RT_SHIFT - 16;
var ADDR_SHIFT = OP_SHIFT - 26;
exports.utils = {
    fromInt: function (instr) {
        var op = (instr & OP_MASK) >>> OP_SHIFT;
        if (op === 0) {
            // R type
            var rs_1 = (instr & RS_MASK) >>> RS_SHIFT;
            var rt_1 = (instr & RT_MASK) >>> RT_SHIFT;
            var rd = (instr & RD_MASK) >>> RD_SHIFT;
            var sh = (instr & SH_MASK) >>> SH_SHIFT;
            var func = (instr & FUNC_MASK) >>> FUNC_SHIFT;
            return new RInstruction(op, rs_1, rt_1, rd, sh, func);
        }
        if (op <= 3) {
            // J type
            var addr = (instr & ADDR_MASK) >>> ADDR_SHIFT;
            return new JInstruction(op, addr);
        }
        var rs = (instr & RS_MASK) >>> RS_SHIFT;
        var rt = (instr & RT_MASK) >>> RT_SHIFT;
        var imm = (instr & IMM_MASK) >>> IMM_SHIFT;
        return new IInstruction(op, rs, rt, imm);
    },
    isR: function (instr) {
        return instr.type === 'R';
    },
    isI: function (instr) {
        return instr.type === 'I';
    },
    isJ: function (instr) {
        return instr.type === 'J';
    },
};
var RInstruction = /** @class */ (function () {
    function RInstruction(op, rs, rt, rd, sh, func) {
        this.type = 'R';
        this.op = op;
        this.rs = rs;
        this.rt = rt;
        this.rd = rd;
        this.sh = sh;
        this.func = func;
    }
    RInstruction.prototype.toInt = function () {
        var _a = this, op = _a.op, rs = _a.rs, rt = _a.rt, rd = _a.rd, sh = _a.sh, func = _a.func;
        return (op << OP_SHIFT) + (rs << RS_SHIFT) + (rt << RT_SHIFT) + (rd << RD_SHIFT) + (sh << SH_SHIFT) + func;
    };
    return RInstruction;
}());
exports.RInstruction = RInstruction;
var IInstruction = /** @class */ (function () {
    function IInstruction(op, rs, rt, imm) {
        this.type = 'I';
        this.op = op;
        this.rs = rs;
        this.rt = rt;
        this.imm = imm;
    }
    IInstruction.prototype.toInt = function () {
        var _a = this, op = _a.op, rs = _a.rs, rt = _a.rt, imm = _a.imm;
        return (op << OP_SHIFT) + (rs << RS_SHIFT) + (rt << RT_SHIFT) + imm;
    };
    return IInstruction;
}());
exports.IInstruction = IInstruction;
var JInstruction = /** @class */ (function () {
    function JInstruction(j, addr) {
        this.type = 'J';
        this.j = j;
        this.addr = addr;
    }
    JInstruction.prototype.toInt = function () {
        var _a = this, j = _a.j, addr = _a.addr;
        return (j << OP_SHIFT) + addr;
    };
    return JInstruction;
}());
exports.JInstruction = JInstruction;
