export declare type NodeType = 'DIRECTIVE' | 'DATA' | 'LABEL' | 'OPERATION' | 'IMMEDIATE' | 'OFFSET' | 'REGISTER' | 'ADDRESS' | 'TRANSFORMED';
export declare class Node {
    type: NodeType;
    children: Node[];
    constructor(type: NodeType);
    add(node: Node): void;
    toString(): string;
}
export declare class DirectiveNode extends Node {
    directiveType: 'data' | 'text';
    constructor(directiveType: 'data' | 'text');
    toString(): string;
}
export declare class DataNode extends Node {
    dataDirective: string;
    values: string[];
    constructor(dataDirective: string, values: string[]);
    toString(): string;
}
export declare class LabelNode extends Node {
    name: string;
    constructor(name: string);
    toString(): string;
}
export declare class RegisterNode extends Node {
    name: string;
    constructor(name: string);
    toString(): string;
}
export declare class ImmediateNode extends Node {
    value: string;
    constructor(value: string);
    split(): ImmediateNode[];
    toString(): string;
}
export declare class OffsetNode extends Node {
    offset: string;
    register: string;
    constructor(offset: string, register: string);
    toString(): string;
}
export declare class AddressNode extends Node {
    label: string;
    partitioned: boolean;
    part: 'upper' | 'lower';
    constructor(label: string);
    partition(part: 'upper' | 'lower'): AddressNode;
    toString(): string;
}
export declare type ArgumentNode = RegisterNode | ImmediateNode | OffsetNode | AddressNode;
export declare class OperationNode extends Node {
    name: string;
    args: ArgumentNode[];
    constructor(name: string, args: ArgumentNode[]);
    toString(): string;
}
export declare class Root {
    data: DirectiveNode;
    text: DirectiveNode;
    constructor();
}
export declare class TransformedNode extends Node {
    original: OperationNode;
    constructor(operations: OperationNode[], original: OperationNode);
    toString(): string;
}
export declare const utils: {
    isOperationNode: (n: Node) => n is OperationNode;
    isLabelNode: (n: Node) => n is LabelNode;
    isDirectiveNode: (n: Node) => n is DirectiveNode;
    isDataNode: (n: Node) => n is DataNode;
    isRegisterNode: (n: Node) => n is RegisterNode;
    isImmediateNode: (n: Node) => n is ImmediateNode;
    isOffsetNode: (n: Node) => n is OffsetNode;
    isAddressNode: (n: Node) => n is AddressNode;
    isTranformedNode: (n: Node) => n is TransformedNode;
};
