"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var parseImmediate_1 = require("../utils/parseImmediate");
var Node = /** @class */ (function () {
    function Node(type) {
        this.children = [];
        this.type = type;
    }
    Node.prototype.add = function (node) {
        this.children.push(node);
    };
    Node.prototype.toString = function () {
        return '[MIPS Node]';
    };
    return Node;
}());
exports.Node = Node;
var DirectiveNode = /** @class */ (function (_super) {
    tslib_1.__extends(DirectiveNode, _super);
    function DirectiveNode(directiveType) {
        var _this = _super.call(this, 'DIRECTIVE') || this;
        _this.directiveType = directiveType;
        return _this;
    }
    DirectiveNode.prototype.toString = function () {
        return "." + this.directiveType;
    };
    return DirectiveNode;
}(Node));
exports.DirectiveNode = DirectiveNode;
var DataNode = /** @class */ (function (_super) {
    tslib_1.__extends(DataNode, _super);
    function DataNode(dataDirective, values) {
        var _this = _super.call(this, 'DATA') || this;
        _this.dataDirective = dataDirective;
        _this.values = values;
        return _this;
    }
    DataNode.prototype.toString = function () {
        return this.values.join(', ');
    };
    return DataNode;
}(Node));
exports.DataNode = DataNode;
var LabelNode = /** @class */ (function (_super) {
    tslib_1.__extends(LabelNode, _super);
    function LabelNode(name) {
        var _this = _super.call(this, 'LABEL') || this;
        _this.name = name;
        return _this;
    }
    LabelNode.prototype.toString = function () {
        return this.name + ":";
    };
    return LabelNode;
}(Node));
exports.LabelNode = LabelNode;
var RegisterNode = /** @class */ (function (_super) {
    tslib_1.__extends(RegisterNode, _super);
    function RegisterNode(name) {
        var _this = _super.call(this, 'REGISTER') || this;
        _this.name = name;
        return _this;
    }
    RegisterNode.prototype.toString = function () {
        return this.name;
    };
    return RegisterNode;
}(Node));
exports.RegisterNode = RegisterNode;
var ImmediateNode = /** @class */ (function (_super) {
    tslib_1.__extends(ImmediateNode, _super);
    function ImmediateNode(value) {
        var _this = _super.call(this, 'IMMEDIATE') || this;
        _this.value = value;
        return _this;
    }
    ImmediateNode.prototype.split = function () {
        var val = parseImmediate_1.default(this.value);
        var upper = (val & 0xffff0000) >>> 16;
        var lower = (val & 0x0000ffff);
        var ret = [];
        if (upper > 0) {
            ret.push(new ImmediateNode(upper.toString()));
        }
        ret.push(new ImmediateNode(lower.toString()));
        return ret;
    };
    ImmediateNode.prototype.toString = function () {
        return this.value;
    };
    return ImmediateNode;
}(Node));
exports.ImmediateNode = ImmediateNode;
var OffsetNode = /** @class */ (function (_super) {
    tslib_1.__extends(OffsetNode, _super);
    function OffsetNode(offset, register) {
        var _this = _super.call(this, 'OFFSET') || this;
        _this.offset = offset;
        _this.register = register;
        return _this;
    }
    OffsetNode.prototype.toString = function () {
        return this.offset + "(" + this.register + ")";
    };
    return OffsetNode;
}(Node));
exports.OffsetNode = OffsetNode;
var AddressNode = /** @class */ (function (_super) {
    tslib_1.__extends(AddressNode, _super);
    function AddressNode(label) {
        var _this = _super.call(this, 'ADDRESS') || this;
        _this.label = label;
        _this.partitioned = false;
        return _this;
    }
    AddressNode.prototype.partition = function (part) {
        var addr = new AddressNode(this.label);
        addr.partitioned = true;
        addr.part = part;
        return addr;
    };
    AddressNode.prototype.toString = function () {
        return this.label;
    };
    return AddressNode;
}(Node));
exports.AddressNode = AddressNode;
var OperationNode = /** @class */ (function (_super) {
    tslib_1.__extends(OperationNode, _super);
    function OperationNode(name, args) {
        var _this = _super.call(this, 'OPERATION') || this;
        _this.name = name;
        _this.args = args;
        return _this;
    }
    OperationNode.prototype.toString = function () {
        return this.name + " " + this.args.map(function (a) { return a.toString(); }).join(', ');
    };
    return OperationNode;
}(Node));
exports.OperationNode = OperationNode;
var Root = /** @class */ (function () {
    function Root() {
        this.data = new DirectiveNode('data');
        this.text = new DirectiveNode('text');
    }
    return Root;
}());
exports.Root = Root;
// Psuedo-instructions will get replaced by these
var TransformedNode = /** @class */ (function (_super) {
    tslib_1.__extends(TransformedNode, _super);
    function TransformedNode(operations, original) {
        var _this = _super.call(this, 'TRANSFORMED') || this;
        _this.children = operations;
        _this.original = original;
        return _this;
    }
    TransformedNode.prototype.toString = function () {
        return this.children.map(function (c) { return c.toString(); }).join('\n');
    };
    return TransformedNode;
}(Node));
exports.TransformedNode = TransformedNode;
exports.utils = {
    isOperationNode: function (n) { return n.type === 'OPERATION'; },
    isLabelNode: function (n) { return n.type === 'LABEL'; },
    isDirectiveNode: function (n) { return n.type === 'DIRECTIVE'; },
    isDataNode: function (n) { return n.type === 'DATA'; },
    isRegisterNode: function (n) { return n.type === 'REGISTER'; },
    isImmediateNode: function (n) { return n.type === 'IMMEDIATE'; },
    isOffsetNode: function (n) { return n.type === 'OFFSET'; },
    isAddressNode: function (n) { return n.type === 'ADDRESS'; },
    isTranformedNode: function (n) { return n.type === 'TRANSFORMED'; },
};
