"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var IO = /** @class */ (function () {
    function IO(_a) {
        var stdin = _a.stdin, stdout = _a.stdout, stderr = _a.stderr;
        this.stdin = stdin;
        this.stdout = stdout;
        this.stderr = stderr;
        this.inBuffer = '';
    }
    IO.prototype.readChar = function () {
        var _this = this;
        if (this.inBuffer.length) {
            var ret = this.inBuffer[0];
            this.inBuffer = this.inBuffer.substr(1);
            return Promise.resolve(ret);
        }
        return new Promise(function (resolve, reject) {
            var removeListeners = function () {
                _this.stdin.removeListener('close', closeHandler);
                _this.stdin.removeListener('error', errHandler);
                _this.stdin.removeListener('data', dataHandler);
            };
            var closeHandler = function () {
                removeListeners();
                reject(new Error("IO: stdin exited during read."));
            };
            var errHandler = function (err) {
                removeListeners();
                reject(err);
            };
            var dataHandler = function (chunk) {
                var data = chunk.toString();
                if (data.length) {
                    var ret = data[0];
                    _this.inBuffer += data.substr(1);
                    removeListeners();
                    resolve(ret);
                }
            };
            _this.stdin.on('close', closeHandler);
            _this.stdin.on('error', errHandler);
            _this.stdin.on('data', dataHandler);
        });
    };
    IO.prototype.readUntilMatch = function (re) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var ret, current;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        ret = '';
                        _a.label = 1;
                    case 1:
                        if (!true) return [3 /*break*/, 3];
                        return [4 /*yield*/, this.readChar()];
                    case 2:
                        current = _a.sent();
                        if (re.test(current)) {
                            return [2 /*return*/, ret];
                        }
                        else {
                            ret += current;
                        }
                        return [3 /*break*/, 1];
                    case 3: return [2 /*return*/];
                }
            });
        });
    };
    // Reads until space
    IO.prototype.read = function () {
        return this.readUntilMatch(/^\s/);
    };
    IO.prototype.readInt = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var str;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.read()];
                    case 1:
                        str = _a.sent();
                        return [2 /*return*/, parseInt(str, 10)];
                }
            });
        });
    };
    IO.prototype.readFloat = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var str;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.read()];
                    case 1:
                        str = _a.sent();
                        return [2 /*return*/, parseFloat(str)];
                }
            });
        });
    };
    IO.prototype.write = function (data) {
        this.stdout.write(data);
    };
    IO.prototype.writeErr = function (data) {
        this.stderr.write(data);
    };
    return IO;
}());
exports.default = IO;
