"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var events_1 = require("events");
var parse_1 = require("../parse");
var assemble_1 = require("../assemble");
var mappings = require("../utils/mappings");
var Instruction_1 = require("../Instruction");
var int32 = require("../utils/int32");
var constants = require("./constants");
var PrimaryRegisters_1 = require("./PrimaryRegisters");
var Memory_1 = require("./Memory");
var Stack_1 = require("./Stack");
var IO_1 = require("./IO");
var OP_CODES = Object.keys(mappings.instructions).reduce(function (opCodes, op) {
    opCodes[op] = mappings.instructions[op].code;
    return opCodes;
}, {});
var MIPS = /** @class */ (function (_super) {
    tslib_1.__extends(MIPS, _super);
    function MIPS(options) {
        var _this = _super.call(this) || this;
        _this.source = options.source;
        _this.io = new IO_1.default({
            stdin: options.stdin,
            stdout: options.stdout,
            stderr: options.stderr,
        });
        return _this;
    }
    MIPS.prototype.updateSource = function (source) {
        this.source = source;
    };
    MIPS.prototype.assemble = function () {
        this.registers = new PrimaryRegisters_1.default();
        this.assembledObj = assemble_1.default(parse_1.default(this.source));
        this.dataMemory = new Memory_1.default({
            baseAddress: constants.BASE_DATA_ADDR,
            size: this.assembledObj.data.length,
            fixed: true,
            initial: this.assembledObj.data,
        });
        this.heapMemory = new Memory_1.default({
            size: 1024,
            baseAddress: constants.BASE_STACK_POINTER + 1,
        });
        this.stack = new Stack_1.default({
            size: 1024,
            end: constants.BASE_STACK_POINTER,
        });
        this.exited = false;
    };
    MIPS.prototype.isAssembled = function () {
        return !!this.assembledObj;
    };
    MIPS.prototype.execute = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var ok;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!!this.exited) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.executeNextInstruction()];
                    case 1:
                        ok = _a.sent();
                        if (!ok)
                            return [3 /*break*/, 2];
                        return [3 /*break*/, 0];
                    case 2: return [2 /*return*/];
                }
            });
        });
    };
    MIPS.prototype.executeNextInstruction = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var index, instr;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        index = this.resolveInstructionIndex();
                        if (index >= this.assembledObj.objInstructions.length) {
                            return [2 /*return*/, false];
                        }
                        instr = this.assembledObj.objInstructions[index];
                        return [4 /*yield*/, this.executeInstruction(instr)];
                    case 1:
                        _a.sent();
                        this.registers.set('pc', this.registers.getUnsigned('pc') + 4);
                        return [2 /*return*/, true];
                }
            });
        });
    };
    MIPS.prototype.executeInstruction = function (i) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var r, a, _a;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        r = this.registers;
                        if (!(i instanceof Instruction_1.RInstruction)) return [3 /*break*/, 28];
                        _a = i.func;
                        switch (_a) {
                            case OP_CODES.addu: return [3 /*break*/, 1];
                            case OP_CODES.add: return [3 /*break*/, 2];
                            case OP_CODES.subu: return [3 /*break*/, 3];
                            case OP_CODES.sub: return [3 /*break*/, 4];
                            case OP_CODES.mult: return [3 /*break*/, 5];
                            case OP_CODES.multu: return [3 /*break*/, 6];
                            case OP_CODES.div: return [3 /*break*/, 7];
                            case OP_CODES.divu: return [3 /*break*/, 8];
                            case OP_CODES.mfhi: return [3 /*break*/, 9];
                            case OP_CODES.mflo: return [3 /*break*/, 10];
                            case OP_CODES.and: return [3 /*break*/, 11];
                            case OP_CODES.or: return [3 /*break*/, 12];
                            case OP_CODES.nor: return [3 /*break*/, 13];
                            case OP_CODES.xor: return [3 /*break*/, 14];
                            case OP_CODES.sll: return [3 /*break*/, 15];
                            case OP_CODES.srl: return [3 /*break*/, 16];
                            case OP_CODES.sra: return [3 /*break*/, 17];
                            case OP_CODES.sllv: return [3 /*break*/, 18];
                            case OP_CODES.srlv: return [3 /*break*/, 19];
                            case OP_CODES.srav: return [3 /*break*/, 20];
                            case OP_CODES.sltu: return [3 /*break*/, 21];
                            case OP_CODES.slt: return [3 /*break*/, 21];
                            case OP_CODES.jr: return [3 /*break*/, 22];
                            case OP_CODES.jalr: return [3 /*break*/, 23];
                            case OP_CODES.syscall: return [3 /*break*/, 24];
                        }
                        return [3 /*break*/, 26];
                    case 1:
                        r.set(i.rd, r.getUnsigned(i.rs) + r.getUnsigned(i.rt));
                        return [3 /*break*/, 27];
                    case 2:
                        r.set(i.rd, r.get(i.rs) + r.get(i.rt));
                        return [3 /*break*/, 27];
                    case 3:
                        r.set(i.rd, int32.unsigned(int32.unsigned(r.get(i.rs)) - int32.unsigned(r.get(i.rt))));
                        return [3 /*break*/, 27];
                    case 4:
                        r.set(i.rd, r.get(i.rs) - r.get(i.rt));
                        return [3 /*break*/, 27];
                    case 5:
                        a = r.get(i.rs) * r.get(i.rt);
                        r.set('hi', (a & 0xffff0000) >>> 16);
                        r.set('lo', a & 0x0000ffff);
                        return [3 /*break*/, 27];
                    case 6:
                        a = int32.unsigned(int32.unsigned(r.get(i.rs)) * int32.unsigned(r.get(i.rt)));
                        r.set('hi', (a & 0xffff0000) >>> 16);
                        r.set('lo', a & 0x0000ffff);
                        return [3 /*break*/, 27];
                    case 7:
                        r.set('lo', (r.get(i.rs) / r.get(i.rt)) | 0);
                        r.set('hi', r.get(i.rs) % r.get(i.rt));
                        return [3 /*break*/, 27];
                    case 8:
                        r.set('lo', int32.unsigned(int32.unsigned(r.get(i.rs)) / int32.unsigned(r.get(i.rt))));
                        r.set('hi', int32.unsigned(int32.unsigned(r.get(i.rs)) % int32.unsigned(r.get(i.rt))));
                        return [3 /*break*/, 27];
                    case 9:
                        r.set(i.rd, r.get('hi'));
                        return [3 /*break*/, 27];
                    case 10:
                        r.set(i.rd, r.get('lo'));
                        return [3 /*break*/, 27];
                    case 11:
                        r.set(i.rd, r.get(i.rs) & r.get(i.rt));
                        return [3 /*break*/, 27];
                    case 12:
                        r.set(i.rd, r.get(i.rs) | r.get(i.rt));
                        return [3 /*break*/, 27];
                    case 13:
                        r.set(i.rd, ~(r.get(i.rs) | r.get(i.rt)));
                        return [3 /*break*/, 27];
                    case 14:
                        r.set(i.rd, r.get(i.rs) ^ r.get(i.rt));
                        return [3 /*break*/, 27];
                    case 15:
                        r.set(i.rd, r.get(i.rt) << i.sh);
                        return [3 /*break*/, 27];
                    case 16:
                        r.set(i.rd, r.get(i.rt) >>> i.sh);
                        return [3 /*break*/, 27];
                    case 17:
                        r.set(i.rd, r.get(i.rt) >> r.get(i.sh));
                        return [3 /*break*/, 27];
                    case 18:
                        r.set(i.rd, r.get(i.rt) << r.get(i.rs));
                        return [3 /*break*/, 27];
                    case 19:
                        r.set(i.rd, r.get(i.rt) >>> r.get(i.rs));
                        return [3 /*break*/, 27];
                    case 20:
                        r.set(i.rd, r.get(i.rt) >> r.get(i.rs));
                        return [3 /*break*/, 27];
                    case 21:
                        r.set(i.rd, r.get(i.rs) < r.get(i.rt) ? 1 : 0);
                        return [3 /*break*/, 27];
                    case 22:
                        r.set('pc', r.getUnsigned(i.rs) - 4);
                        return [3 /*break*/, 27];
                    case 23:
                        r.set('$ra', r.getUnsigned('pc') + 4);
                        r.set('pc', r.getUnsigned(i.rs));
                        return [3 /*break*/, 27];
                    case 24: return [4 /*yield*/, this.syscall(r.get('$v0'))];
                    case 25:
                        _b.sent();
                        return [3 /*break*/, 27];
                    case 26: throw new Error("Unknown R-type: " + i.func);
                    case 27: return [3 /*break*/, 29];
                    case 28:
                        if (i instanceof Instruction_1.IInstruction) {
                            switch (i.op) {
                                case OP_CODES.addi:
                                    r.set(i.rt, r.get(i.rs) + i.imm);
                                    break;
                                case OP_CODES.addiu:
                                    r.set(i.rt, int32.unsigned(int32.unsigned(r.get(i.rs)) + int32.unsigned(i.imm)));
                                    break;
                                case OP_CODES.andi:
                                    r.set(i.rt, r.get(i.rs) & i.imm);
                                    break;
                                case OP_CODES.ori:
                                    r.set(i.rt, r.get(i.rs) | i.imm);
                                    break;
                                case OP_CODES.xori:
                                    r.set(i.rt, r.get(i.rs) ^ i.imm);
                                    break;
                                case OP_CODES.slti:
                                    r.set(i.rt, int32.signed(r.get(i.rs)) < int32.signed(i.imm) ? 1 : 0);
                                    break;
                                case OP_CODES.sltiu:
                                    r.set(i.rt, int32.unsigned(r.get(i.rs)) < int32.unsigned(i.imm) ? 1 : 0);
                                    break;
                                case OP_CODES.beq:
                                    if (int32.signed(r.get(i.rs)) === int32.signed(r.get(i.rt))) {
                                        r.set('pc', r.get('pc') + (i.imm << 2));
                                    }
                                    break;
                                case OP_CODES.bne:
                                    if (int32.signed(r.get(i.rs)) !== int32.signed(r.get(i.rt))) {
                                        r.set('pc', r.get('pc') + (i.imm << 2));
                                    }
                                    break;
                                case OP_CODES.lui:
                                    r.set(i.rt, i.imm << 16);
                                    break;
                                case OP_CODES.lb:
                                    r.set(i.rt, int32.signed(this.readByte(r.get(i.rs) + i.imm) & 0x000000ff));
                                    break;
                                case OP_CODES.lbu:
                                    r.set(i.rt, int32.unsigned(this.readByte(r.get(i.rs) + i.imm) & 0x000000ff));
                                    break;
                                case OP_CODES.lh:
                                    r.set(i.rt, int32.signed(this.readHalf(r.get(i.rs) + i.imm) & 0x0000ffff));
                                    break;
                                case OP_CODES.lhu:
                                    r.set(i.rt, int32.unsigned(this.readHalf(r.get(i.rs) + i.imm) & 0x0000ffff));
                                    break;
                                case OP_CODES.lw:
                                    r.set(i.rt, int32.signed(this.readWord(r.get(i.rs) + i.imm)));
                                    break;
                                case OP_CODES.sb:
                                    this.writeByte(r.getUnsigned(i.rs) + i.imm, r.get(i.rt));
                                    break;
                                case OP_CODES.sh:
                                    this.writeHalf(r.getUnsigned(i.rs) + i.imm, r.get(i.rt));
                                    break;
                                case OP_CODES.sw:
                                    this.writeWord(r.getUnsigned(i.rs) + i.imm, r.get(i.rt));
                                    break;
                                case OP_CODES.ll:
                                    r.set(i.rt, this.readWord(r.get(i.rs) + i.imm));
                                    break;
                                case OP_CODES.sc:
                                    // TODO: Doube check implementation
                                    this.writeWord(r.get(i.rs) + i.imm, r.get(i.rt));
                                    r.set(i.rt, 1);
                                    break;
                                default:
                                    throw new Error("Unknow I instruction: " + i.op);
                            }
                        }
                        else if (i instanceof Instruction_1.JInstruction) {
                            // console.log('Executing (J): ', findCode(i.j, 'J'))
                            switch (i.j) {
                                case OP_CODES.j:
                                    r.set('pc', i.addr - 4);
                                    break;
                                case OP_CODES.jal:
                                    r.set('$ra', r.get('pc') + 4);
                                    r.set('pc', i.addr - 4);
                                    break;
                            }
                        }
                        _b.label = 29;
                    case 29: return [2 /*return*/];
                }
            });
        });
    };
    MIPS.prototype.resolveInstructionIndex = function () {
        var pc = this.registers.get('pc');
        return (pc - constants.BASE_TEXT_ADDR) / 4;
    };
    MIPS.prototype.resolveMemory = function (address) {
        var begOfData = constants.BASE_DATA_ADDR;
        var endOfData = constants.BASE_DATA_ADDR + this.dataMemory.getSize();
        var endOfStack = constants.BASE_STACK_POINTER;
        if (address >= begOfData && address < endOfData) {
            return this.dataMemory;
        }
        if (address < endOfStack) {
            return this.stack;
        }
        return this.heapMemory;
    };
    MIPS.prototype.readByte = function (address) {
        var mem = this.resolveMemory(address);
        return mem.get(address);
    };
    MIPS.prototype.writeByte = function (address, value) {
        var mem = this.resolveMemory(address);
        mem.set(address, value);
    };
    MIPS.prototype.readHalf = function (address) {
        var mem = this.resolveMemory(address);
        var ret = 0;
        ret += mem.get(address + 0) << 8;
        ret += mem.get(address + 1);
        return ret;
    };
    MIPS.prototype.writeHalf = function (address, value) {
        var mem = this.resolveMemory(address);
        mem.set(address + 0, (value & 0x0000ff00) >>> 8);
        mem.set(address + 1, (value & 0x000000ff));
    };
    MIPS.prototype.readWord = function (address) {
        var mem = this.resolveMemory(address);
        var ret = 0;
        ret += mem.get(address + 0) << 24;
        ret += mem.get(address + 1) << 16;
        ret += mem.get(address + 2) << 8;
        ret += mem.get(address + 3);
        return ret | 0;
    };
    MIPS.prototype.writeWord = function (address, value) {
        var mem = this.resolveMemory(address);
        mem.set(address + 0, (value & 0xff000000) >>> 24);
        mem.set(address + 1, (value & 0x00ff0000) >>> 16);
        mem.set(address + 2, (value & 0x0000ff00) >>> 8);
        mem.set(address + 3, (value & 0x000000ff));
    };
    MIPS.prototype.readASCIIString = function (address) {
        var ret = '';
        var c;
        while ((c = this.readByte(address)) !== 0) {
            ret += String.fromCharCode(c);
            address++;
        }
        return ret;
    };
    MIPS.prototype.syscall = function (code) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var r, _a, v;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        r = this.registers;
                        _a = code;
                        switch (_a) {
                            case 1: return [3 /*break*/, 1];
                            case 2: return [3 /*break*/, 2];
                            case 3: return [3 /*break*/, 2];
                            case 4: return [3 /*break*/, 3];
                            case 5: return [3 /*break*/, 4];
                            case 6: return [3 /*break*/, 6];
                            case 7: return [3 /*break*/, 6];
                            case 8: return [3 /*break*/, 6];
                            case 9: return [3 /*break*/, 6];
                            case 10: return [3 /*break*/, 7];
                        }
                        return [3 /*break*/, 8];
                    case 1:
                        // print integer
                        this.io.write(r.get('$a0').toString());
                        return [3 /*break*/, 8];
                    case 2: 
                    // TODO: print float, double
                    return [3 /*break*/, 8];
                    case 3:
                        // print string
                        this.io.write(this.readASCIIString(r.get('$a0')));
                        return [3 /*break*/, 8];
                    case 4: return [4 /*yield*/, this.io.readInt()
                        // TODO: Handle NaN case
                    ];
                    case 5:
                        v = _b.sent();
                        // TODO: Handle NaN case
                        r.set('$v0', v);
                        return [3 /*break*/, 8];
                    case 6: 
                    // TODO: read float, double, string. alloc heap
                    return [3 /*break*/, 8];
                    case 7:
                        this.exited = true;
                        return [3 /*break*/, 8];
                    case 8: return [2 /*return*/];
                }
            });
        });
    };
    return MIPS;
}(events_1.EventEmitter));
exports.default = MIPS;
