"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var events_1 = require("events");
var Memory = /** @class */ (function (_super) {
    tslib_1.__extends(Memory, _super);
    function Memory(_a) {
        var size = _a.size, initial = _a.initial, _b = _a.fixed, fixed = _b === void 0 ? false : _b, _c = _a.baseAddress, baseAddress = _c === void 0 ? 0 : _c;
        var _this = _super.call(this) || this;
        _this.size = size;
        _this.data = new Uint8Array(_this.size);
        _this.fixed = fixed;
        _this.baseAddress = baseAddress;
        if (initial) {
            _this.clone(initial);
        }
        return _this;
    }
    Memory.prototype.clone = function (data) {
        for (var i = 0; i < data.length; i++) {
            if (i > this.data.length - 1) {
                throw new Error("Memory: Clone failed, out of range");
            }
            this.data[i] = data[i];
        }
    };
    Memory.prototype.resolveIndex = function (address) {
        return address - this.baseAddress;
    };
    Memory.prototype.resolveAddress = function (index) {
        return index + this.baseAddress;
    };
    Memory.prototype.expand = function (size) {
        var oldData = this.data;
        this.data = new Uint8Array(size);
        this.size = size;
        this.clone(oldData);
    };
    Memory.prototype.get = function (address) {
        // TODO: Check bounds?
        return this.data[this.resolveIndex(address)];
    };
    Memory.prototype.set = function (address, value) {
        var prevValue = this.get(address);
        var event = {
            address: address,
            prevValue: prevValue,
            nextValue: value,
        };
        var index = this.resolveIndex(address);
        if (index >= this.data.length) {
            this.expand(this.size * 2);
        }
        this.data[index] = value;
        this.emit('change', event);
    };
    Memory.prototype.getSize = function () {
        return this.size;
    };
    return Memory;
}(events_1.EventEmitter));
exports.default = Memory;
