"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var events_1 = require("events");
var invertIndexTable_1 = require("../utils/invertIndexTable");
var int32 = require("../utils/int32");
var Registers = /** @class */ (function (_super) {
    tslib_1.__extends(Registers, _super);
    function Registers(options) {
        var _this = _super.call(this) || this;
        _this.size = options.size;
        _this.nameToIndex = options.mappings;
        _this.indexToName = invertIndexTable_1.default(options.mappings);
        _this.registers = new Uint32Array(_this.size);
        return _this;
    }
    Registers.prototype.set = function (register, value) {
        var name = typeof register === 'string' ? register : this.indexToName[register];
        var index = typeof register === 'number' ? register : this.nameToIndex[register];
        if (typeof name === 'undefined' || typeof index === 'undefined') {
            throw new Error("Registers: Register \"" + register + "\" not found.");
        }
        var prevValue = this.registers[index];
        var nextValue = value;
        this.registers[index] = nextValue;
        this.emit('change', {
            name: name,
            index: index,
            prevValue: prevValue,
            nextValue: nextValue,
        });
    };
    Registers.prototype.get = function (register, signed) {
        if (signed === void 0) { signed = true; }
        var index = typeof register === 'number' ? register : this.nameToIndex[register];
        if (typeof index === 'undefined' || index > this.size - 1) {
            throw new Error("Registers: Register \"" + register + "\" is out of range.");
        }
        var value = this.registers[index];
        return signed ? int32.signed(value) : int32.unsigned(value);
    };
    Registers.prototype.getUnsigned = function (register) {
        return this.get(register, false);
    };
    return Registers;
}(events_1.EventEmitter));
exports.default = Registers;
